package cz.muni.fi.pb162.project.geometry;

import java.util.Arrays;

/**
 * Class for working with vertices of polygons
 *
 * @author Vitalii Bortsov
 */
public class ArrayPolygon extends SimplePolygon {
    private final Vertex2D[] arrayOfVertices;

    /**
     * Constructor of the array with vertices of the polygon
     *
     * @param vertices coordinates of the vertices of the polygon;
     *                        must not be null,
     *                        must not contain null-element,
     *                        must has at least 3 elements.
     */
    public ArrayPolygon(Vertex2D[] vertices) {
        if(vertices == null) {
            throw new IllegalArgumentException("Array must not be null");
        }
        if(Arrays.asList(vertices).contains(null)) {
            throw new IllegalArgumentException("Array must not contain a null element");
        }
        if(vertices.length < 3) {
            throw new IllegalArgumentException("Array must has at least 3 elements");
        }

        arrayOfVertices = Arrays.copyOf(vertices, vertices.length);
    }

    @Override
    public Vertex2D getVertex(int index) {
        if(index < 0) {
            throw new IllegalArgumentException("index must be positive");
        }

        return arrayOfVertices[index % arrayOfVertices.length];
    }

    @Override
    public int getNumVertices() {
        return arrayOfVertices.length;
    }

    @Override
    public boolean equals(Object object) {
        if(object == null) {
            return false;
        }
        if(this.getClass() != object.getClass()) {
            return false;
        }

        ArrayPolygon secondArray = (ArrayPolygon) object;
        final int length = this.arrayOfVertices.length;
        if (length != secondArray.arrayOfVertices.length) {
            return false;
        }
        for (int i = 0; i < length; i++) {
            if(!this.arrayOfVertices[i].equals(secondArray.arrayOfVertices[i])) {
                return false;
            }
        }
        return true;
    }

    @Override
    public int hashCode() {
        return Arrays.hashCode(arrayOfVertices);
    }
}
